/*
  Copyright (c) 2013, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "mmss.hpp"

#include <cstring>


namespace MMSS {

void dispatch(const std::shared_ptr<packet_t> &packet) {
	auto source = packet->source.lock();
	auto dest = packet->dest.lock();
	if (!source || !dest)
		return;

	auto node = dest->node.lock();
	if (!node)
		return;

	node->proto->handle_packet(node.get(), node->ctx, dest.get(), &source->address, packet->data.get(), packet->len);
}

void add_iface(const std::shared_ptr<node_t> &node, const std::shared_ptr<network_t> &net, const std::string &name, const gmrf_addr_t *address) {
	std::shared_ptr<iface_t> iface = iface_t::create();

	iface->name = name;
	iface->address = *address;

	iface->node = node;
	iface->net = net;

	node->interfaces.push_back(iface);
	net->interfaces.push_back(iface);

	node->proto->add_iface(node.get(), node->ctx, iface.get());
}

void enqueue(context_t *mmss, const std::shared_ptr<iface_t> &source, const std::shared_ptr<iface_t> &dest, const void *data, size_t len) {
	std::shared_ptr<packet_t> packet = std::make_shared<packet_t>();

	packet->sent = mmss->now;
	packet->source = source;
	packet->dest = dest;
	packet->len = len;

	packet->data.reset(new uint8_t[len]);
	std::memcpy(packet->data.get(), data, len);

	mmss->packet_queue.put(packet, mmss->now+1);
}

}
